/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.commons.chain2.impl;

/**
 * Subclass of <code>ContextBase</code> to exercise the automatic
 * delegation to properties of the <code>Context</code> class.
 *
 * @version $Id$
 */
public class TestContext extends ContextBase {

    /** serialVersionUID */
    private static final long serialVersionUID = 20120724L;

    /** Read-write property */
    private String readWrite = "readWrite";

    /** Write-only property */
    private String writeOnly = "writeOnly";

    /**
     * @return String
     */
    public String getReadOnly() {
        return "readOnly";
    }

    /**
     * @return String
     */
    public String getReadWrite() {
        return this.readWrite;
    }

    /**
     * @param val String
     */
    public void setReadWrite(final String val) {
        this.readWrite = val;
    }

    /**
     *  Not a JavaBeans getter
     * @return String
     */
    public String returnWriteOnly() {
        return this.writeOnly;
    }

    /**
     * @param val String
     */
    public void setWriteOnly(final String val) {
        this.writeOnly = val;
    }

    /**
     * @see org.apache.commons.chain2.impl.ContextBase#equals(java.lang.Object)
     */
    @Override
    public boolean equals(final Object o) {
        return (o instanceof TestContext) && super.equals(o);
    }

    /**
     * @see org.apache.commons.chain2.impl.ContextBase#hashCode()
     */
    @Override
    public int hashCode() {
        return super.hashCode() + 0;
    }
}
