package project.check.range;

import java.util.Optional;

import core.util.NumberUtil;
import core.util.bean.Pair;
import online.context.check.ItemCheck;
import online.model.ModelUtil;
import project.check.BaseCheck;

/**
 * 指定範囲内チェック
 *
 * @author Tadashi Nakayama
 */
public final class NumeralRangeCheck extends BaseCheck implements ItemCheck {

	/** 最大数 */
	private final long maxval;
	/** 最小数 */
	private final long minval;
	/** メッセージID */
	private final String mid;
	/** メッセージパラメータ */
	private final String[] prm;

	/**
	 * コンストラクタ
	 *
	 * @param min 指定最小文字数
	 * @param max 指定最大文字数
	 */
	public NumeralRangeCheck(final long min, final long max) {
		this.minval = min;
		this.maxval = max;
		this.mid = null;
		this.prm = null;
	}

	/**
	 * コンストラクタ
	 *
	 * @param min 指定最小文字数
	 * @param max 指定最大文字数
	 * @param msgId メッセージID
	 * @param param メッセージパラメータ
	 */
	public NumeralRangeCheck(final long min, final long max,
			final String msgId, final String... param) {
		this.minval = min;
		this.maxval = max;
		this.mid = msgId;
		this.prm = extend(param, String.valueOf(max), String.valueOf(min));
	}

	/**
	 * @see online.context.check.ItemCheck#check(java.lang.String...)
	 */
	@Override
	public void check(final String... items) {
		Optional<Pair<String, Integer>> check = Optional.empty();
		for (final var item : items) {
			final var val = getArrayParameter(item);
			for (var j = 0; val != null && j < val.length; j++) {
				if (!isTarget(j)) {
					val[j] = ModelUtil.getValueAsString(getUniModel(), item, j);
					continue;
				}
				final var num = NumberUtil.toLong(val[j]);
				if (num != null && (num > this.maxval || num < this.minval)) {
					setMessage(item, j, this.mid, this.prm);
					check = Optional.of(check.orElse(Pair.of(item, j)));
				}
			}
		}

		check.ifPresent(this::throwKeepCheckException);
	}
}
